@@ -2,16 +2,18 @@
 '''
 Manage the information in the hosts file
 '''
-from __future__ import absolute_import
 
 # Import python libs
+from __future__ import absolute_import
 import os
 
 # Import salt libs
 import salt.utils
 import salt.utils.odict as odict
-from salt.ext.six.moves import range
+
+# Import 3rd-party libs
 import salt.ext.six as six
+from salt.ext.six.moves import range  # pylint: disable=import-error,no-name-in-module,redefined-builtin
 
 
 # pylint: disable=C0103
@@ -19,10 +21,6 @@ def __get_hosts_filename():
     '''
     Return the path to the appropriate hosts file
     '''
-    # TODO: Investigate using  "%SystemRoot%\system32" for this
-    if salt.utils.is_windows():
-        return 'C:\\Windows\\System32\\drivers\\etc\\hosts'
-
     return __salt__['config.option']('hosts.file')
 
 
@@ -35,7 +33,8 @@ def _get_or_create_hostfile():
     if hfn is None:
         hfn = ''
     if not os.path.exists(hfn):
-        salt.utils.fopen(hfn, 'w').close()
+        with salt.utils.fopen(hfn, 'w'):
+            pass
     return hfn
 
 
@@ -106,6 +105,10 @@ def get_alias(ip):
     '''
     Return the list of aliases associated with an ip
 
+    Aliases (host names) are returned in the order in which they
+    appear in the hosts file.  If there are no aliases associated with
+    the IP, an empty list is returned.
+
     CLI Example:
 
     .. code-block:: bash
@@ -137,6 +140,11 @@ def set_host(ip, alias):
     Set the host entry in the hosts file for the given ip, this will overwrite
     any previous entry for the given ip
 
+    .. versionchanged:: 2016.3.0
+        If ``alias`` does not include any host names (it is the empty
+        string or contains only whitespace), all entries for the given
+        IP address are removed.
+
     CLI Example:
 
     .. code-block:: bash
@@ -147,7 +155,14 @@ def set_host(ip, alias):
     ovr = False
     if not os.path.isfile(hfn):
         return False
-    lines = salt.utils.fopen(hfn).readlines()
+
+    line_to_add = ip + '\t\t' + alias + os.linesep
+    # support removing a host entry by providing an empty string
+    if not alias.strip():
+        line_to_add = ''
+
+    with salt.utils.fopen(hfn) as fp_:
+        lines = fp_.readlines()
     for ind, line in enumerate(lines):
         tmpline = line.strip()
         if not tmpline:
@@ -157,15 +172,15 @@ def set_host(ip, alias):
         comps = tmpline.split()
         if comps[0] == ip:
             if not ovr:
-                lines[ind] = ip + '\t\t' + alias + '\n'
+                lines[ind] = line_to_add
                 ovr = True
             else:  # remove other entries
                 lines[ind] = ''
     if not ovr:
         # make sure there is a newline
-        if lines and not lines[-1].endswith(('\n', '\r')):
-            lines[-1] = '{0}\n'.format(lines[-1])
-        line = ip + '\t\t' + alias + '\n'
+        if lines and not lines[-1].endswith(os.linesep):
+            lines[-1] += os.linesep
+        line = line_to_add
         lines.append(line)
     with salt.utils.fopen(hfn, 'w+') as ofile:
         ofile.writelines(lines)
@@ -185,7 +200,8 @@ def rm_host(ip, alias):
     if not has_pair(ip, alias):
         return True
     hfn = _get_or_create_hostfile()
-    lines = salt.utils.fopen(hfn).readlines()
+    with salt.utils.fopen(hfn) as fp_:
+        lines = fp_.readlines()
     for ind in range(len(lines)):
         tmpline = lines[ind].strip()
         if not tmpline:
@@ -194,17 +210,17 @@ def rm_host(ip, alias):
             continue
         comps = tmpline.split()
         if comps[0] == ip:
-            newline = '{0}\t'.format(comps[0])
+            newline = '{0}\t\t'.format(comps[0])
             for existing in comps[1:]:
                 if existing == alias:
                     continue
-                newline += '\t{0}'.format(existing)
+                newline += ' {0}'.format(existing)
             if newline.strip() == ip:
                 # No aliases exist for the line, make it empty
                 lines[ind] = ''
             else:
                 # Only an alias was removed
-                lines[ind] = '{0}\n'.format(newline)
+                lines[ind] = newline + os.linesep
     with salt.utils.fopen(hfn, 'w+') as ofile:
         ofile.writelines(lines)
     return True
@@ -230,7 +246,7 @@ def add_host(ip, alias):
 
     hosts = _list_hosts()
     inserted = False
-    for i, h in hosts.items():
+    for i, h in six.iteritems(hosts):
         for j in range(len(h)):
             if h[j].startswith('#') and i == ip:
                 h.insert(j, alias)
@@ -250,7 +266,7 @@ def _write_hosts(hosts):
             else:
                 line = '{0}\t\t{1}'.format(
                     ip,
-                    '\t\t'.join(aliases)
+                    ' '.join(aliases)
                     )
         lines.append(line)
 
@@ -260,4 +276,4 @@ def _write_hosts(hosts):
             if line.strip():
                 # /etc/hosts needs to end with EOL so that some utils that read
                 # it do not break
-                ofile.write('{0}\n'.format(line.strip()))
+                ofile.write(line.strip() + os.linesep)
