@@ -1,20 +1,55 @@
 # -*- coding: utf-8 -*-
 '''
 Manage users with the useradd command
+
+.. important::
+    If you feel that Salt should be using this module to manage users on a
+    minion, and it is using a different module (or gives an error similar to
+    *'user.info' is not available*), see :ref:`here
+    <module-provider-override>`.
 '''
 
+# Notes:
+# ------
+#
+# Format of the master.passwd file:
+#
+# - name      User's login name.
+# - password  User's encrypted password.
+# - uid       User's id.
+# - gid       User's login group id.
+# - class     User's login class.
+# - change    Password change time.
+# - expire    Account expiration time.
+# - gecos     General information about the user.
+# - home_dir  User's home directory.
+# - shell     User's login shell.
+#
+# The usershow command allows viewing of an account in a format that is
+# identical to the format used in /etc/master.passwd (with the password field
+# replaced with a ‘*’.)
+#
+# Example:
+# % pw usershow -n someuser
+# someuser:*:1001:1001::0:0:SomeUser Name:/home/someuser:/bin/sh
+
 # Import python libs
+from __future__ import absolute_import
+import copy
+import logging
 try:
     import pwd
+    HAS_PWD = True
 except ImportError:
-    pass
-import logging
-import copy
+    HAS_PWD = False
+
+# Import 3rd party libs
+import salt.ext.six as six
 
 # Import salt libs
 import salt.utils
-from salt.ext.six import string_types
 from salt.exceptions import CommandExecutionError
+from salt.utils import locales
 
 log = logging.getLogger(__name__)
 
@@ -24,26 +59,33 @@ __virtualname__ = 'user'
 
 def __virtual__():
     '''
-    Set the user module if the kernel is FreeBSD
+    Set the user module if the kernel is FreeBSD or DragonFly
     '''
-    return __virtualname__ if __grains__['kernel'] == 'FreeBSD' else False
+    if HAS_PWD and __grains__['kernel'] in ('FreeBSD', 'DragonFly'):
+        return __virtualname__
+    return (False, 'The pw_user execution module cannot be loaded: the pwd python module is not available or the system is not FreeBSD.')
 
 
 def _get_gecos(name):
     '''
     Retrieve GECOS field info and return it in dictionary form
     '''
-    gecos_field = pwd.getpwnam(name).pw_gecos.split(',', 3)
+    try:
+        gecos_field = pwd.getpwnam(name).pw_gecos.split(',', 3)
+    except KeyError:
+        raise CommandExecutionError(
+            'User \'{0}\' does not exist'.format(name)
+        )
     if not gecos_field:
         return {}
     else:
         # Assign empty strings for any unspecified trailing GECOS fields
         while len(gecos_field) < 4:
             gecos_field.append('')
-        return {'fullname': str(gecos_field[0]),
-                'roomnumber': str(gecos_field[1]),
-                'workphone': str(gecos_field[2]),
-                'homephone': str(gecos_field[3])}
+        return {'fullname': locales.sdecode(gecos_field[0]),
+                'roomnumber': locales.sdecode(gecos_field[1]),
+                'workphone': locales.sdecode(gecos_field[2]),
+                'homephone': locales.sdecode(gecos_field[3])}
 
 
 def _build_gecos(gecos_dict):
@@ -51,12 +93,31 @@ def _build_gecos(gecos_dict):
     Accepts a dictionary entry containing GECOS field names and their values,
     and returns a full GECOS comment string, to be used with pw usermod.
     '''
-    return '{0},{1},{2},{3}'.format(gecos_dict.get('fullname', ''),
+    return u'{0},{1},{2},{3}'.format(gecos_dict.get('fullname', ''),
                                     gecos_dict.get('roomnumber', ''),
                                     gecos_dict.get('workphone', ''),
                                     gecos_dict.get('homephone', ''))
 
 
+def _update_gecos(name, key, value):
+    '''
+    Common code to change a user's GECOS information
+    '''
+    if not isinstance(value, six.string_types):
+        value = str(value)
+    pre_info = _get_gecos(name)
+    if not pre_info:
+        return False
+    if value == pre_info[key]:
+        return True
+    gecos_data = copy.deepcopy(pre_info)
+    gecos_data[key] = value
+    cmd = ['pw', 'usermod', name, '-c', _build_gecos(gecos_data)]
+    __salt__['cmd.run'](cmd, python_shell=False)
+    post_info = info(name)
+    return _get_gecos(name).get(key) == value
+
+
 def add(name,
         uid=None,
         gid=None,
@@ -69,6 +130,7 @@ def add(name,
         workphone='',
         homephone='',
         createhome=True,
+        loginclass=None,
         **kwargs):
     '''
     Add a user to the minion
@@ -85,32 +147,32 @@ def add(name,
     if kwargs:
         log.warning('Invalid kwargs passed to user.add')
 
-    if isinstance(groups, string_types):
+    if isinstance(groups, six.string_types):
         groups = groups.split(',')
-    cmd = 'pw useradd '
+    cmd = ['pw', 'useradd']
     if uid:
-        cmd += '-u {0} '.format(uid)
+        cmd.extend(['-u', uid])
     if gid:
-        cmd += '-g {0} '.format(gid)
+        cmd.extend(['-g', gid])
     if groups:
-        cmd += '-G {0} '.format(','.join(groups))
+        cmd.extend(['-G', ','.join(groups)])
     if home is not None:
-        cmd += '-d {0} '.format(home)
+        cmd.extend(['-d', home])
     if createhome is True:
-        cmd += '-m '
+        cmd.append('-m')
+    if loginclass:
+        cmd.extend(['-L', loginclass])
     if shell:
-        cmd += '-s {0} '.format(shell)
+        cmd.extend(['-s', shell])
     if not salt.utils.is_true(unique):
-        cmd += '-o '
-    gecos_field = '{0},{1},{2},{3}'.format(fullname,
-                                           roomnumber,
-                                           workphone,
-                                           homephone)
-    cmd += '-c "{0}" '.format(gecos_field)
-    cmd += '-n {0}'.format(name)
-    ret = __salt__['cmd.run_all'](cmd, python_shell=False)
-
-    return not ret['retcode']
+        cmd.append('-o')
+    gecos_field = _build_gecos({'fullname': fullname,
+                                'roomnumber': roomnumber,
+                                'workphone': workphone,
+                                'homephone': homephone})
+    cmd.extend(['-c', gecos_field])
+    cmd.extend(['-n', name])
+    return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
 
 
 def delete(name, remove=False, force=False):
@@ -126,14 +188,11 @@ def delete(name, remove=False, force=False):
     if salt.utils.is_true(force):
         log.error('pw userdel does not support force-deleting user while '
                   'user is logged in')
-    cmd = 'pw userdel '
+    cmd = ['pw', 'userdel']
     if remove:
-        cmd += '-r '
-    cmd += '-n ' + name
-
-    ret = __salt__['cmd.run_all'](cmd, python_shell=False)
-
-    return not ret['retcode']
+        cmd.append('-r')
+    cmd.extend(['-n', name])
+    return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
 
 
 def getent(refresh=False):
@@ -167,14 +226,15 @@ def chuid(name, uid):
         salt '*' user.chuid foo 4376
     '''
     pre_info = info(name)
+    if not pre_info:
+        raise CommandExecutionError(
+            'User \'{0}\' does not exist'.format(name)
+        )
     if uid == pre_info['uid']:
         return True
-    cmd = 'pw usermod -u {0} -n {1}'.format(uid, name)
+    cmd = ['pw', 'usermod', '-u', uid, '-n', name]
     __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['uid'] != pre_info['uid']:
-        return post_info['uid'] == uid
-    return False
+    return info(name).get('uid') == uid
 
 
 def chgid(name, gid):
@@ -188,14 +248,15 @@ def chgid(name, gid):
         salt '*' user.chgid foo 4376
     '''
     pre_info = info(name)
+    if not pre_info:
+        raise CommandExecutionError(
+            'User \'{0}\' does not exist'.format(name)
+        )
     if gid == pre_info['gid']:
         return True
-    cmd = 'pw usermod -g {0} -n {1}'.format(gid, name)
+    cmd = ['pw', 'usermod', '-g', gid, '-n', name]
     __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['gid'] != pre_info['gid']:
-        return post_info['gid'] == gid
-    return False
+    return info(name).get('gid') == gid
 
 
 def chshell(name, shell):
@@ -209,20 +270,31 @@ def chshell(name, shell):
         salt '*' user.chshell foo /bin/zsh
     '''
     pre_info = info(name)
+    if not pre_info:
+        raise CommandExecutionError(
+            'User \'{0}\' does not exist'.format(name)
+        )
     if shell == pre_info['shell']:
         return True
-    cmd = 'pw usermod -s {0} -n {1}'.format(shell, name)
+    cmd = ['pw', 'usermod', '-s', shell, '-n', name]
     __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['shell'] != pre_info['shell']:
-        return post_info['shell'] == shell
-    return False
+    return info(name).get('shell') == shell
 
 
 def chhome(name, home, persist=False):
     '''
-    Change the home directory of the user, pass true for persist to copy files
-    to the new home dir
+    Set a new home directory for an existing user
+
+    name
+        Username to modify
+
+    home
+        New home directory to set
+
+    persist : False
+        Set to ``True`` to prevent configuration files in the new home
+        directory from being overwritten by the files from the skeleton
+        directory.
 
     CLI Example:
 
@@ -231,22 +303,34 @@ def chhome(name, home, persist=False):
         salt '*' user.chhome foo /home/users/foo True
     '''
     pre_info = info(name)
+    if not pre_info:
+        raise CommandExecutionError(
+            'User \'{0}\' does not exist'.format(name)
+        )
     if home == pre_info['home']:
         return True
-    cmd = 'pw usermod {0} -d {1}'.format(name, home)
+    cmd = ['pw', 'usermod', name, '-d', home]
     if persist:
-        cmd += ' -m '
+        cmd.append('-m')
     __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['home'] != pre_info['home']:
-        return post_info['home'] == home
-    return False
+    return info(name).get('home') == home
 
 
 def chgroups(name, groups, append=False):
     '''
-    Change the groups this user belongs to, add append to append the specified
+    Change the groups to which a user belongs
+
+    name
+        Username to modify
+
     groups
+        List of groups to set for the user. Can be passed as a comma-separated
+        list or a Python list.
+
+    append : False
+        Set to ``True`` to append these groups to the user's existing list of
+        groups. Otherwise, the specified groups will replace any existing
+        groups for the user.
 
     CLI Example:
 
@@ -254,15 +338,15 @@ def chgroups(name, groups, append=False):
 
         salt '*' user.chgroups foo wheel,root True
     '''
-    if isinstance(groups, string_types):
+    if isinstance(groups, six.string_types):
         groups = groups.split(',')
     ugrps = set(list_groups(name))
     if ugrps == set(groups):
         return True
     if append:
         groups += ugrps
-    cmd = 'pw usermod -G {0} -n {1}'.format(','.join(groups), name)
-    return not __salt__['cmd.retcode'](cmd, python_shell=False)
+    cmd = ['pw', 'usermod', '-G', ','.join(groups), '-n', name]
+    return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
 
 
 def chfullname(name, fullname):
@@ -275,20 +359,7 @@ def chfullname(name, fullname):
 
         salt '*' user.chfullname foo "Foo Bar"
     '''
-    fullname = str(fullname)
-    pre_info = _get_gecos(name)
-    if not pre_info:
-        return False
-    if fullname == pre_info['fullname']:
-        return True
-    gecos_field = copy.deepcopy(pre_info)
-    gecos_field['fullname'] = fullname
-    cmd = 'pw usermod {0} -c "{1}"'.format(name, _build_gecos(gecos_field))
-    __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['fullname'] != pre_info['fullname']:
-        return post_info['fullname'] == fullname
-    return False
+    return _update_gecos(name, 'fullname', fullname)
 
 
 def chroomnumber(name, roomnumber):
@@ -301,20 +372,7 @@ def chroomnumber(name, roomnumber):
 
         salt '*' user.chroomnumber foo 123
     '''
-    roomnumber = str(roomnumber)
-    pre_info = _get_gecos(name)
-    if not pre_info:
-        return False
-    if roomnumber == pre_info['roomnumber']:
-        return True
-    gecos_field = copy.deepcopy(pre_info)
-    gecos_field['roomnumber'] = roomnumber
-    cmd = 'pw usermod {0} -c "{1}"'.format(name, _build_gecos(gecos_field))
-    __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['roomnumber'] != pre_info['roomnumber']:
-        return post_info['roomnumber'] == roomnumber
-    return False
+    return _update_gecos(name, 'roomnumber', roomnumber)
 
 
 def chworkphone(name, workphone):
@@ -327,20 +385,7 @@ def chworkphone(name, workphone):
 
         salt '*' user.chworkphone foo "7735550123"
     '''
-    workphone = str(workphone)
-    pre_info = _get_gecos(name)
-    if not pre_info:
-        return False
-    if workphone == pre_info['workphone']:
-        return True
-    gecos_field = copy.deepcopy(pre_info)
-    gecos_field['workphone'] = workphone
-    cmd = 'pw usermod {0} -c "{1}"'.format(name, _build_gecos(gecos_field))
-    __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['workphone'] != pre_info['workphone']:
-        return post_info['workphone'] == workphone
-    return False
+    return _update_gecos(name, 'workphone', workphone)
 
 
 def chhomephone(name, homephone):
@@ -353,20 +398,30 @@ def chhomephone(name, homephone):
 
         salt '*' user.chhomephone foo "7735551234"
     '''
-    homephone = str(homephone)
-    pre_info = _get_gecos(name)
-    if not pre_info:
-        return False
-    if homephone == pre_info['homephone']:
+    return _update_gecos(name, 'homephone', homephone)
+
+
+def chloginclass(name, loginclass, root=None):
+    '''
+    Change the default login class of the user
+
+    .. versionadded:: 2016.3.5
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' user.chloginclass foo staff
+    '''
+    if loginclass == get_loginclass(name):
         return True
-    gecos_field = copy.deepcopy(pre_info)
-    gecos_field['homephone'] = homephone
-    cmd = 'pw usermod {0} -c "{1}"'.format(name, _build_gecos(gecos_field))
+
+    cmd = ['pw', 'usermod', '-L', '{0}'.format(loginclass),
+           '-n', '{0}'.format(name)]
+
     __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['homephone'] != pre_info['homephone']:
-        return post_info['homephone'] == homephone
-    return False
+
+    return get_loginclass(name) == loginclass
 
 
 def info(name):
@@ -403,6 +458,26 @@ def info(name):
     return ret
 
 
+def get_loginclass(name):
+    '''
+    Get the login class of the user
+
+    .. versionadded:: 2016.3.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' user.get_loginclass foo
+
+    '''
+
+    userinfo = __salt__['cmd.run_stdout'](['pw', 'usershow', '-n', name])
+    userinfo = userinfo.split(':')
+
+    return {'loginclass': userinfo[4] if len(userinfo) == 10 else ''}
+
+
 def list_groups(name):
     '''
     Return a list of groups the named user belongs to
@@ -441,11 +516,13 @@ def rename(name, new_name):
     '''
     current_info = info(name)
     if not current_info:
-        raise CommandExecutionError('User {0!r} does not exist'.format(name))
+        raise CommandExecutionError('User \'{0}\' does not exist'.format(name))
     new_info = info(new_name)
     if new_info:
-        raise CommandExecutionError('User {0!r} already exists'.format(new_name))
-    cmd = 'pw usermod -l {0} -n {1}'.format(new_name, name)
+        raise CommandExecutionError(
+            'User \'{0}\' already exists'.format(new_name)
+        )
+    cmd = ['pw', 'usermod', '-l', new_name, '-n', name]
     __salt__['cmd.run'](cmd)
     post_info = info(new_name)
     if post_info['name'] != current_info['name']:
