@@ -27,11 +27,12 @@ import hashlib  # do not remove, used in imported file.py functions
 import errno  # do not remove, used in imported file.py functions
 import shutil  # do not remove, used in imported file.py functions
 import re  # do not remove, used in imported file.py functions
+import string  # do not remove, used in imported file.py functions
 import sys  # do not remove, used in imported file.py functions
 import fileinput  # do not remove, used in imported file.py functions
 import fnmatch  # do not remove, used in imported file.py functions
 import mmap  # do not remove, used in imported file.py functions
-from salt.ext.six import string_types  # do not remove, used in imported file.py functions
+import glob  # do not remove, used in imported file.py functions
 # do not remove, used in imported file.py functions
 import salt.ext.six as six  # pylint: disable=import-error,no-name-in-module
 from salt.ext.six.moves.urllib.parse import urlparse as _urlparse  # pylint: disable=import-error,no-name-in-module
@@ -55,15 +56,15 @@ import salt.utils
 from salt.modules.file import (check_hash,  # pylint: disable=W0611
         directory_exists, get_managed, mkdir, makedirs_, makedirs_perms,
         check_managed, check_managed_changes, check_perms, source_list,
-        touch, append, contains, contains_regex, contains_regex_multiline,
+        touch, append, contains, contains_regex, get_source_sum,
         contains_glob, find, psed, get_sum, _get_bkroot, _mkstemp_copy,
-        get_hash, manage_file, file_exists, get_diff, list_backups,
+        get_hash, manage_file, file_exists, get_diff, line, list_backups,
         __clean_tmp, check_file_meta, _binary_replace, restore_backup,
         access, copy, readdir, rmdir, truncate, replace, delete_backup,
         search, _get_flags, extract_hash, _error, _sed_esc, _psed,
         RE_FLAG_TABLE, blockreplace, prepend, seek_read, seek_write, rename,
-        lstat, path_exists_glob, write, pardir, join, HASHES, comment,
-        uncomment, _add_flags, comment_line)
+        lstat, path_exists_glob, write, pardir, join, HASHES, HASHES_REVMAP,
+        comment, uncomment, _add_flags, comment_line, apply_template_on_contents)
 
 from salt.utils import namespaced_function as _namespaced_function
 
@@ -83,13 +84,13 @@ def __virtual__():
             global source_list, mkdir, __clean_tmp, makedirs_, file_exists
             global check_managed, check_managed_changes, check_file_meta
             global append, _error, directory_exists, touch, contains
-            global contains_regex, contains_regex_multiline, contains_glob
+            global contains_regex, contains_glob, get_source_sum
             global find, psed, get_sum, check_hash, get_hash, delete_backup
             global get_diff, _get_flags, extract_hash, comment_line
             global access, copy, readdir, rmdir, truncate, replace, search
             global _binary_replace, _get_bkroot, list_backups, restore_backup
             global blockreplace, prepend, seek_read, seek_write, rename, lstat
-            global write, pardir, join, _add_flags
+            global write, pardir, join, _add_flags, apply_template_on_contents
             global path_exists_glob, comment, uncomment, _mkstemp_copy
 
             replace = _namespaced_function(replace, globals())
@@ -119,8 +120,8 @@ def __virtual__():
             touch = _namespaced_function(touch, globals())
             contains = _namespaced_function(contains, globals())
             contains_regex = _namespaced_function(contains_regex, globals())
-            contains_regex_multiline = _namespaced_function(contains_regex_multiline, globals())
             contains_glob = _namespaced_function(contains_glob, globals())
+            get_source_sum = _namespaced_function(get_source_sum, globals())
             find = _namespaced_function(find, globals())
             psed = _namespaced_function(psed, globals())
             get_sum = _namespaced_function(get_sum, globals())
@@ -147,9 +148,10 @@ def __virtual__():
             comment_line = _namespaced_function(comment_line, globals())
             _mkstemp_copy = _namespaced_function(_mkstemp_copy, globals())
             _add_flags = _namespaced_function(_add_flags, globals())
+            apply_template_on_contents = _namespaced_function(apply_template_on_contents, globals())
 
             return __virtualname__
-    return False
+    return (False, "Module win_file: module only works on Windows systems")
 
 __outputter__ = {
     'touch': 'txt',
@@ -842,7 +844,7 @@ def chgrp(path, group):
     return None
 
 
-def stats(path, hash_type='md5', follow_symlinks=True):
+def stats(path, hash_type='sha256', follow_symlinks=True):
     '''
     Return a dict containing the stats for a given file
 
@@ -1101,7 +1103,7 @@ def remove(path, force=False):
             # Reset attributes to the original if delete fails.
             win32api.SetFileAttributes(path, file_attributes)
         raise CommandExecutionError(
-            'Could not remove {0!r}: {1}'.format(path, exc)
+            'Could not remove \'{0}\': {1}'.format(path, exc)
         )
 
     return True
@@ -1146,7 +1148,11 @@ def symlink(src, link):
         return True
     except pywinerror as exc:
         raise CommandExecutionError(
-            'Could not create {0!r} - [{1}] {2}'.format(link, exc.winerror, exc.strerror)
+            'Could not create \'{0}\' - [{1}] {2}'.format(
+                link,
+                exc.winerror,
+                exc.strerror
+            )
         )
 
 
@@ -1170,7 +1176,7 @@ def _is_reparse_point(path):
 
 def is_link(path):
     '''
-    Return the path that a symlink points to
+    Check if the path is a symlink
 
     This is only supported on Windows Vista or later.
 
