@@ -6,13 +6,17 @@ from __future__ import absolute_import
 
 # Import python libs
 import re
+import logging
 
 # Import salt libs
 import salt.utils
+from salt.ext import six
 
 # Define the module's virtual name
 __virtualname__ = 'firewall'
 
+log = logging.getLogger(__name__)
+
 
 def __virtual__():
     '''
@@ -20,7 +24,7 @@ def __virtual__():
     '''
     if salt.utils.is_windows():
         return __virtualname__
-    return False
+    return (False, "Module win_firewall: module only works on Windows systems")
 
 
 def get_config():
@@ -51,7 +55,7 @@ def get_config():
 
 def disable(profile='allprofiles'):
     '''
-    Disable all the firewall profiles
+    Disable firewall profile :param profile: (default: allprofiles)
 
     CLI Example:
 
@@ -59,7 +63,7 @@ def disable(profile='allprofiles'):
 
         salt '*' firewall.disable
     '''
-    cmd = ['netsh', 'advfirewall', 'set', 'allprofiles', 'state', 'off']
+    cmd = ['netsh', 'advfirewall', 'set', profile, 'state', 'off']
     return __salt__['cmd.run'](cmd, python_shell=False) == 'Ok.'
 
 
@@ -75,12 +79,11 @@ def enable(profile='allprofiles'):
 
         salt '*' firewall.enable
     '''
-    return __salt__['cmd.run'](
-            'netsh advfirewall set {0} state on'.format(profile)
-            ) == 'Ok.'
+    cmd = ['netsh', 'advfirewall', 'set', profile, 'state', 'on']
+    return __salt__['cmd.run'](cmd, python_shell=False) == 'Ok.'
 
 
-def get_rule(name="all"):
+def get_rule(name='all'):
     '''
     .. versionadded:: 2015.5.0
 
@@ -90,19 +93,20 @@ def get_rule(name="all"):
 
     .. code-block:: bash
 
-        salt '*' firewall.get_rule "MyAppPort"
+        salt '*' firewall.get_rule 'MyAppPort'
     '''
     ret = {}
     cmd = ['netsh', 'advfirewall', 'firewall', 'show', 'rule', 'name={0}'.format(name)]
     ret[name] = __salt__['cmd.run'](cmd, python_shell=False)
 
-    if ret[name].strip() == "No rules match the specified criteria.":
+    if ret[name].strip() == 'No rules match the specified criteria.':
         ret = False
 
     return ret
 
 
-def add_rule(name, localport, protocol="tcp", action="allow", dir="in"):
+def add_rule(name, localport, protocol='tcp', action='allow', dir='in',
+             remoteip='any'):
     '''
     .. versionadded:: 2015.5.0
 
@@ -112,12 +116,54 @@ def add_rule(name, localport, protocol="tcp", action="allow", dir="in"):
 
     .. code-block:: bash
 
-        salt '*' firewall.add_rule "test" "tcp" "8080"
+        salt '*' firewall.add_rule 'test' '8080' 'tcp'
+        salt '*' firewall.add_rule 'test' '1' 'icmpv4'
+        salt '*' firewall.add_rule 'test_remote_ip' '8000' 'tcp' 'allow' 'in' '192.168.0.1'
+
     '''
     cmd = ['netsh', 'advfirewall', 'firewall', 'add', 'rule',
            'name={0}'.format(name),
            'protocol={0}'.format(protocol),
            'dir={0}'.format(dir),
-           'localport={0}'.format(localport),
-           'action={0}'.format(action)]
-    return __salt__['cmd.run'](cmd, python_shell=False) == 'Ok.'
+           'action={0}'.format(action),
+           'remoteip={0}'.format(remoteip)]
+
+    if 'icmpv4' not in protocol and 'icmpv6' not in protocol:
+        cmd.append('localport={0}'.format(localport))
+
+    ret = __salt__['cmd.run'](cmd, python_shell=False)
+    if isinstance(ret, six.string_types):
+        return ret.strip() == 'Ok.'
+    else:
+        log.error('firewall.add_rule failed: {0}'.format(ret))
+        return False
+
+
+def delete_rule(name, localport, protocol='tcp', dir='in', remoteip='any'):
+    '''
+    .. versionadded:: 2015.8.0
+
+    Delete an existing firewall rule
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' firewall.delete_rule 'test' '8080' 'tcp' 'in'
+        salt '*' firewall.delete_rule 'test_remote_ip' '8000' 'tcp' 'in' '192.168.0.1'
+    '''
+    cmd = ['netsh', 'advfirewall', 'firewall', 'delete', 'rule',
+           'name={0}'.format(name),
+           'protocol={0}'.format(protocol),
+           'dir={0}'.format(dir),
+           'remoteip={0}'.format(remoteip)]
+
+    if 'icmpv4' not in protocol and 'icmpv6' not in protocol:
+        cmd.append('localport={0}'.format(localport))
+
+    ret = __salt__['cmd.run'](cmd, python_shell=False)
+    if isinstance(ret, six.string_types):
+        return ret.endswith('Ok.')
+    else:
+        log.error('firewall.delete_rule failed: {0}'.format(ret))
+        return False
