@@ -11,13 +11,12 @@
 from __future__ import absolute_import
 import os
 import logging
-import multiprocessing
 import signal
-import tempfile
 from threading import Thread, Event
 
 # Import salt libs
 import salt.log
+import salt.cache
 import salt.client
 import salt.pillar
 import salt.utils
@@ -27,6 +26,7 @@ import salt.payload
 from salt.exceptions import SaltException
 import salt.config
 from salt.utils.cache import CacheCli as cache_cli
+from salt.utils.process import MultiprocessingProcess
 
 # Import third party libs
 import salt.ext.six as six
@@ -72,17 +72,7 @@ class MasterPillarUtil(object):
                  use_cached_pillar=True,
                  grains_fallback=True,
                  pillar_fallback=True,
-                 opts=None,
-                 env=None):
-        if env is not None:
-            salt.utils.warn_until(
-                'Boron',
-                'Passing a salt environment should be done using \'saltenv\' '
-                'not \'env\'. This functionality will be removed in Salt '
-                'Boron.'
-            )
-            # Backwards compatibility
-            saltenv = env
+                 opts=None):
 
         log.debug('New instance of {0} created.'.format(
             self.__class__.__name__))
@@ -101,8 +91,9 @@ class MasterPillarUtil(object):
         self.use_cached_pillar = use_cached_pillar
         self.grains_fallback = grains_fallback
         self.pillar_fallback = pillar_fallback
+        self.cache = salt.cache.Cache(opts)
         log.debug(
-            'Init settings: tgt: {0!r}, expr_form: {1!r}, saltenv: {2!r}, '
+            'Init settings: tgt: \'{0}\', expr_form: \'{1}\', saltenv: \'{2}\', '
             'use_cached_grains: {3}, use_cached_pillar: {4}, '
             'grains_fallback: {5}, pillar_fallback: {6}'.format(
                 tgt, expr_form, saltenv, use_cached_grains, use_cached_pillar,
@@ -118,19 +109,14 @@ class MasterPillarUtil(object):
             log.debug('Skipping cached mine data minion_data_cache'
                       'and enfore_mine_cache are both disabled.')
             return mine_data
-        mdir = os.path.join(self.opts['cachedir'], 'minions')
-        try:
-            for minion_id in minion_ids:
-                if not salt.utils.verify.valid_id(self.opts, minion_id):
-                    continue
-                path = os.path.join(mdir, minion_id, 'mine.p')
-                if os.path.isfile(path):
-                    with salt.utils.fopen(path, 'rb') as fp_:
-                        mdata = self.serial.loads(fp_.read())
-                        if isinstance(mdata, dict):
-                            mine_data[minion_id] = mdata
-        except (OSError, IOError):
-            return mine_data
+        if not minion_ids:
+            minion_ids = self.cache.list('minions')
+        for minion_id in minion_ids:
+            if not salt.utils.verify.valid_id(self.opts, minion_id):
+                continue
+            mdata = self.cache.fetch('minions/{0}'.format(minion_id), 'mine')
+            if isinstance(mdata, dict):
+                mine_data[minion_id] = mdata
         return mine_data
 
     def _get_cached_minion_data(self, *minion_ids):
@@ -142,21 +128,16 @@ class MasterPillarUtil(object):
             log.debug('Skipping cached data because minion_data_cache is not '
                       'enabled.')
             return grains, pillars
-        mdir = os.path.join(self.opts['cachedir'], 'minions')
-        try:
-            for minion_id in minion_ids:
-                if not salt.utils.verify.valid_id(self.opts, minion_id):
-                    continue
-                path = os.path.join(mdir, minion_id, 'data.p')
-                if os.path.isfile(path):
-                    with salt.utils.fopen(path, 'rb') as fp_:
-                        mdata = self.serial.loads(fp_.read())
-                        if mdata.get('grains', False):
-                            grains[minion_id] = mdata['grains']
-                        if mdata.get('pillar', False):
-                            pillars[minion_id] = mdata['pillar']
-        except (OSError, IOError):
-            return grains, pillars
+        if not minion_ids:
+            minion_ids = self.cache.list('minions')
+        for minion_id in minion_ids:
+            if not salt.utils.verify.valid_id(self.opts, minion_id):
+                continue
+            mdata = self.cache.fetch('minions/{0}'.format(minion_id), 'data')
+            if 'grains' in mdata:
+                grains[minion_id] = mdata['grains']
+            if 'pillar' in mdata:
+                pillars[minion_id] = mdata['pillar']
         return grains, pillars
 
     def _get_live_minion_grains(self, minion_ids):
@@ -175,7 +156,7 @@ class MasterPillarUtil(object):
         if minion_id is None:
             return {}
         if not minion_grains:
-            log.warn(
+            log.warning(
                 'Cannot get pillar data for {0}: no grains supplied.'.format(
                     minion_id
                 )
@@ -343,7 +324,7 @@ class MasterPillarUtil(object):
         if clear_mine:
             clear_what.append('mine')
         if clear_mine_func is not None:
-            clear_what.append('mine_func: {0!r}'.format(clear_mine_func))
+            clear_what.append('mine_func: \'{0}\''.format(clear_mine_func))
         if not len(clear_what):
             log.debug('No cached data types specified for clearing.')
             return False
@@ -363,50 +344,35 @@ class MasterPillarUtil(object):
             # in the same file, 'data.p'
             grains, pillars = self._get_cached_minion_data(*minion_ids)
         try:
+            c_minions = self.cache.list('minions')
             for minion_id in minion_ids:
                 if not salt.utils.verify.valid_id(self.opts, minion_id):
                     continue
-                cdir = os.path.join(self.opts['cachedir'], 'minions', minion_id)
-                if not os.path.isdir(cdir):
-                    # Cache dir for this minion does not exist. Nothing to do.
+
+                if minion_id not in c_minions:
+                    # Cache bank for this minion does not exist. Nothing to do.
                     continue
-                data_file = os.path.join(cdir, 'data.p')
-                mine_file = os.path.join(cdir, 'mine.p')
+                bank = 'minions/{0}'.format(minion_id)
                 minion_pillar = pillars.pop(minion_id, False)
                 minion_grains = grains.pop(minion_id, False)
                 if ((clear_pillar and clear_grains) or
                     (clear_pillar and not minion_grains) or
                     (clear_grains and not minion_pillar)):
                     # Not saving pillar or grains, so just delete the cache file
-                    os.remove(os.path.join(data_file))
+                    self.cache.flush(bank, 'data')
                 elif clear_pillar and minion_grains:
-                    tmpfh, tmpfname = tempfile.mkstemp(dir=cdir)
-                    os.close(tmpfh)
-                    with salt.utils.fopen(tmpfname, 'w+b') as fp_:
-                        fp_.write(self.serial.dumps({'grains': minion_grains}))
-                    salt.utils.atomicfile.atomic_rename(tmpfname, data_file)
+                    self.cache.store(bank, 'data', {'grains': minion_grains})
                 elif clear_grains and minion_pillar:
-                    tmpfh, tmpfname = tempfile.mkstemp(dir=cdir)
-                    os.close(tmpfh)
-                    with salt.utils.fopen(tmpfname, 'w+b') as fp_:
-                        fp_.write(self.serial.dumps({'pillar': minion_pillar}))
-                    salt.utils.atomicfile.atomic_rename(tmpfname, data_file)
+                    self.cache.store(bank, 'data', {'pillar': minion_pillar})
                 if clear_mine:
                     # Delete the whole mine file
-                    os.remove(os.path.join(mine_file))
+                    self.cache.flush(bank, 'mine')
                 elif clear_mine_func is not None:
                     # Delete a specific function from the mine file
-                    with salt.utils.fopen(mine_file, 'rb') as fp_:
-                        mine_data = self.serial.loads(fp_.read())
+                    mine_data = self.cache.fetch(bank, 'mine')
                     if isinstance(mine_data, dict):
                         if mine_data.pop(clear_mine_func, False):
-                            tmpfh, tmpfname = tempfile.mkstemp(dir=cdir)
-                            os.close(tmpfh)
-                            with salt.utils.fopen(tmpfname, 'w+b') as fp_:
-                                fp_.write(self.serial.dumps(mine_data))
-                            salt.utils.atomicfile.atomic_rename(
-                                tmpfname,
-                                mine_file)
+                            self.cache.store(bank, 'mine', mine_data)
         except (OSError, IOError):
             return True
         return True
@@ -445,20 +411,31 @@ class CacheTimer(Thread):
                 count = 0
 
 
-class CacheWorker(multiprocessing.Process):
+class CacheWorker(MultiprocessingProcess):
     '''
     Worker for ConnectedCache which runs in its
     own process to prevent blocking of ConnectedCache
     main-loop when refreshing minion-list
     '''
 
-    def __init__(self, opts):
+    def __init__(self, opts, log_queue=None):
         '''
         Sets up the zmq-connection to the ConCache
         '''
-        super(CacheWorker, self).__init__()
+        super(CacheWorker, self).__init__(log_queue=log_queue)
         self.opts = opts
 
+    # __setstate__ and __getstate__ are only used on Windows.
+    # We do this so that __init__ will be invoked on Windows in the child
+    # process so that a register_after_fork() equivalent will work on Windows.
+    def __setstate__(self, state):
+        self._is_child = True
+        self.__init__(state['opts'], log_queue=state['log_queue'])
+
+    def __getstate__(self):
+        return {'opts': self.opts,
+                'log_queue': self.log_queue}
+
     def run(self):
         '''
         Gather currently connected minions and update the cache
@@ -470,7 +447,7 @@ class CacheWorker(multiprocessing.Process):
         log.debug('ConCache CacheWorker update finished')
 
 
-class ConnectedCache(multiprocessing.Process):
+class ConnectedCache(MultiprocessingProcess):
     '''
     Provides access to all minions ids that the master has
     successfully authenticated. The cache is cleaned up regularly by
@@ -478,11 +455,11 @@ class ConnectedCache(multiprocessing.Process):
     the master publisher port.
     '''
 
-    def __init__(self, opts):
+    def __init__(self, opts, log_queue=None):
         '''
         starts the timer and inits the cache itself
         '''
-        super(ConnectedCache, self).__init__()
+        super(ConnectedCache, self).__init__(log_queue=log_queue)
         log.debug('ConCache initializing...')
 
         # the possible settings for the cache
@@ -504,6 +481,17 @@ class ConnectedCache(multiprocessing.Process):
         self.timer.start()
         self.running = True
 
+    # __setstate__ and __getstate__ are only used on Windows.
+    # We do this so that __init__ will be invoked on Windows in the child
+    # process so that a register_after_fork() equivalent will work on Windows.
+    def __setstate__(self, state):
+        self._is_child = True
+        self.__init__(state['opts'], log_queue=state['log_queue'])
+
+    def __getstate__(self):
+        return {'opts': self.opts,
+                'log_queue': self.log_queue}
+
     def signal_handler(self, sig, frame):
         '''
         handle signals and shutdown
@@ -668,8 +656,13 @@ class ConnectedCache(multiprocessing.Process):
 
 def ping_all_connected_minions(opts):
     client = salt.client.LocalClient()
-    ckminions = salt.utils.minions.CkMinions(opts)
-    client.cmd(list(ckminions.connected_ids()), 'test.ping', expr_form='list')
+    if opts['minion_data_cache']:
+        tgt = list(salt.utils.minions.CkMinions(opts).connected_ids())
+        form = 'list'
+    else:
+        tgt = '*'
+        form = 'glob'
+    client.cmd(tgt, 'test.ping', expr_form=form)
 
 # test code for the ConCache class
 if __name__ == '__main__':
