@@ -1,20 +1,37 @@
 # -*- coding: utf-8 -*-
 
-# Import salt libs
-from salt.utils.winservice import Service, instart
-import salt
-import salt.exitcodes
-
-# Import third party libs
-import win32serviceutil
-import win32service
-import winerror
-
 # Import python libs
+from __future__ import absolute_import
 import sys
 
+# Import salt libs
+from salt.utils.winservice import service, instart
+import salt
+import salt.defaults.exitcodes
 
-class MinionService(Service):
+# Import third party libs
+try:
+    import win32serviceutil
+    import win32service
+    import winerror
+    HAS_WIN32 = True
+except ImportError:
+    HAS_WIN32 = False
+
+
+# Although utils are often directly imported, it is also possible to use the
+# loader.
+def __virtual__():
+    '''
+    Only load if Win32 Libraries are installed
+    '''
+    if not HAS_WIN32:
+        return False, 'This utility requires pywin32'
+
+    return 'saltminionservice'
+
+
+class MinionService(service(False)):
 
     def start(self):
         self.runflag = True
@@ -38,7 +55,7 @@ def _main():
     except win32service.error as details:
         if details[0] == winerror.ERROR_SERVICE_DOES_NOT_EXIST:
             instart(MinionService, servicename, 'Salt Minion')
-            sys.exit(salt.exitcodes.EX_OK)
+            sys.exit(salt.defaults.exitcodes.EX_OK)
     if status[1] == win32service.SERVICE_RUNNING:
         win32serviceutil.StopServiceWithDeps(servicename)
         win32serviceutil.StartService(servicename)
