@@ -21,12 +21,11 @@ import jinja2.ext
 # Import salt libs
 import salt.utils
 import salt.utils.yamlencoding
+import salt.utils.locales
 from salt.exceptions import (
     SaltRenderError, CommandExecutionError, SaltInvocationError
 )
-from salt.utils.jinja import ensure_sequence_filter, show_full_context
-from salt.utils.jinja import SaltCacheLoader as JinjaSaltCacheLoader
-from salt.utils.jinja import SerializerExtension as JinjaSerializerExtension
+import salt.utils.jinja
 from salt.utils.odict import OrderedDict
 from salt import __path__ as saltpath
 from salt.ext.six import string_types
@@ -41,6 +40,68 @@ TEMPLATE_DIRNAME = os.path.join(saltpath[0], 'templates')
 SLS_ENCODING = 'utf-8'  # this one has no BOM.
 SLS_ENCODER = codecs.getencoder(SLS_ENCODING)
 
+ALIAS_WARN = (
+        'Starting in 2015.5, cmd.run uses python_shell=False by default, '
+        'which doesn\'t support shellisms (pipes, env variables, etc). '
+        'cmd.run is currently aliased to cmd.shell to prevent breakage. '
+        'Please switch to cmd.shell or set python_shell=True to avoid '
+        'breakage in the future, when this aliasing is removed.'
+)
+ALIASES = {
+        'cmd.run': 'cmd.shell',
+        'cmd': {'run': 'shell'},
+}
+
+
+class AliasedLoader(object):
+    '''
+    Light wrapper around the LazyLoader to redirect 'cmd.run' calls to
+    'cmd.shell', for easy use of shellisms during templating calls
+
+    Dotted aliases ('cmd.run') must resolve to another dotted alias
+    (e.g. 'cmd.shell')
+
+    Non-dotted aliases ('cmd') must resolve to a dictionary of function
+    aliases for that module (e.g. {'run': 'shell'})
+    '''
+
+    def __init__(self, wrapped):
+        self.wrapped = wrapped
+
+    def __getitem__(self, name):
+        if name in ALIASES:
+            salt.utils.warn_until('Nitrogen', ALIAS_WARN)
+            return self.wrapped[ALIASES[name]]
+        else:
+            return self.wrapped[name]
+
+    def __getattr__(self, name):
+        if name in ALIASES:
+            salt.utils.warn_until('Nitrogen', ALIAS_WARN)
+            return AliasedModule(getattr(self.wrapped, name), ALIASES[name])
+        else:
+            return getattr(self.wrapped, name)
+
+
+class AliasedModule(object):
+    '''
+    Light wrapper around module objects returned by the LazyLoader's getattr
+    for the purposes of `salt.cmd.run()` syntax in templates
+
+    Allows for aliasing specific functions, such as `run` to `shell` for easy
+    use of shellisms during templating calls
+    '''
+    def __init__(self, wrapped, aliases):
+        self.aliases = aliases
+        self.wrapped = wrapped
+
+    def __getattr__(self, name):
+        if name in self.aliases:
+            salt.utils.warn_until('Nitrogen', ALIAS_WARN)
+            return getattr(self.wrapped, self.aliases[name])
+        else:
+            return getattr(self.wrapped, name)
+
 
 def wrap_tmpl_func(render_str):
 
@@ -57,11 +118,7 @@ def wrap_tmpl_func(render_str):
         # Alias cmd.run to cmd.shell to make python_shell=True the default for
         # templated calls
         if 'salt' in kws:
-            if 'cmd.run' in kws['salt'] and 'cmd.shell' in kws['salt']:
-                kws['salt']['cmd.run'] = kws['salt']['cmd.shell']
-            if 'run' in kws['salt'].get('cmd', {}) \
-                    and 'shell' in kws['salt'].get('cmd', {}):
-                kws['salt']['cmd']['run'] = kws['salt']['cmd']['shell']
+            kws['salt'] = AliasedLoader(kws['salt'])
 
         # We want explicit context to overwrite the **kws
         kws.update(context)
@@ -82,7 +139,7 @@ def wrap_tmpl_func(render_str):
                 tpldir = os.path.dirname(template).replace('\\', '/')
                 tpldata = {
                     'tplfile': template,
-                    'tpldir': tpldir,
+                    'tpldir': '.' if tpldir == '' else tpldir,
                     'tpldot': tpldir.replace('/', '.'),
                 }
                 context.update(tpldata)
@@ -124,6 +181,7 @@ def wrap_tmpl_func(render_str):
                 output = os.linesep.join(output.splitlines())
 
         except SaltRenderError as exc:
+            log.error("Rendering exception occurred: {0}".format(exc))
             #return dict(result=False, data=str(exc))
             raise
         except Exception:
@@ -131,7 +189,7 @@ def wrap_tmpl_func(render_str):
         else:
             if to_str:  # then render as string
                 return dict(result=True, data=output)
-            with tempfile.NamedTemporaryFile('wb', delete=False) as outf:
+            with tempfile.NamedTemporaryFile('wb', delete=False, prefix=salt.utils.files.TEMPFILE_PREFIX) as outf:
                 outf.write(SLS_ENCODER(output)[0])
                 # Note: If nothing is replaced or added by the rendering
                 #       function, then the contents of the output file will
@@ -220,8 +278,10 @@ def _get_jinja_error(trace, context=None):
     if add_log:
         if template_path:
             out = '\n{0}\n'.format(msg.splitlines()[0])
+            with salt.utils.fopen(template_path) as fp_:
+                template_contents = fp_.read()
             out += salt.utils.get_context(
-                salt.utils.fopen(template_path).read(),
+                template_contents,
                 line,
                 marker='    <======================')
         else:
@@ -254,7 +314,7 @@ def render_jinja_tmpl(tmplstr, context, tmplpath=None):
             loader = jinja2.FileSystemLoader(
                 context, os.path.dirname(tmplpath))
     else:
-        loader = JinjaSaltCacheLoader(opts, saltenv, pillar_rend=context.get('_pillar_rend', False))
+        loader = salt.utils.jinja.SaltCacheLoader(opts, saltenv, pillar_rend=context.get('_pillar_rend', False))
 
     env_args = {'extensions': [], 'loader': loader}
 
@@ -264,7 +324,7 @@ def render_jinja_tmpl(tmplstr, context, tmplpath=None):
         env_args['extensions'].append('jinja2.ext.do')
     if hasattr(jinja2.ext, 'loopcontrols'):
         env_args['extensions'].append('jinja2.ext.loopcontrols')
-    env_args['extensions'].append(JinjaSerializerExtension)
+    env_args['extensions'].append(salt.utils.jinja.SerializerExtension)
 
     # Pass through trim_blocks and lstrip_blocks Jinja parameters
     # trim_blocks removes newlines around Jinja blocks
@@ -284,13 +344,15 @@ def render_jinja_tmpl(tmplstr, context, tmplpath=None):
                                        **env_args)
 
     jinja_env.filters['strftime'] = salt.utils.date_format
-    jinja_env.filters['sequence'] = ensure_sequence_filter
+    jinja_env.filters['sequence'] = salt.utils.jinja.ensure_sequence_filter
     jinja_env.filters['yaml_dquote'] = salt.utils.yamlencoding.yaml_dquote
     jinja_env.filters['yaml_squote'] = salt.utils.yamlencoding.yaml_squote
     jinja_env.filters['yaml_encode'] = salt.utils.yamlencoding.yaml_encode
 
     jinja_env.globals['odict'] = OrderedDict
-    jinja_env.globals['show_full_context'] = show_full_context
+    jinja_env.globals['show_full_context'] = salt.utils.jinja.show_full_context
+
+    jinja_env.tests['list'] = salt.utils.is_list
 
     decoded_context = {}
     for key, value in six.iteritems(context):
@@ -298,7 +360,7 @@ def render_jinja_tmpl(tmplstr, context, tmplpath=None):
             decoded_context[key] = value
             continue
 
-        decoded_context[key] = salt.utils.sdecode(value)
+        decoded_context[key] = salt.utils.locales.sdecode(value)
 
     try:
         template = jinja_env.from_string(tmplstr)
@@ -340,6 +402,13 @@ def render_jinja_tmpl(tmplstr, context, tmplpath=None):
             tmplstr = ''
         else:
             tmplstr += '\n{0}'.format(tracestr)
+        log.debug("Jinja Error")
+        log.debug("Exception: {0}".format(exc))
+        log.debug("Out: {0}".format(out))
+        log.debug("Line: {0}".format(line))
+        log.debug("TmplStr: {0}".format(tmplstr))
+        log.debug("TraceStr: {0}".format(tracestr))
+
         raise SaltRenderError('Jinja error: {0}{1}'.format(exc, out),
                               line,
                               tmplstr,
@@ -366,7 +435,10 @@ def render_mako_tmpl(tmplstr, context, tmplpath=None):
             from mako.lookup import TemplateLookup
             lookup = TemplateLookup(directories=[os.path.dirname(tmplpath)])
     else:
-        lookup = SaltMakoTemplateLookup(context['opts'], saltenv)
+        lookup = SaltMakoTemplateLookup(
+                context['opts'],
+                saltenv,
+                pillar_rend=context.get('_pillar_rend', False))
     try:
         return Template(
             tmplstr,
